module Elasticsearch
  module XPack
    module API
      module Watcher
        module Actions; end

        class WatcherClient
          include Elasticsearch::API::Common::Client, Elasticsearch::API::Common::Client::Base, Watcher::Actions
        end

        def watcher
          @watcher ||= WatcherClient.new(self)
        end

      end
    end
  end
end
